/**
 * @license
 * Copyright 2025 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
/**
 * Normalizes HTTP header values by handling multiline values.
 * Multiline header values are joined with commas according to HTTP/1.1 spec.
 *
 * @internal
 */
export declare function normalizeHeaderValue(name: string, value: string): string;
//# sourceMappingURL=httpUtils.d.ts.map