/**
 * @license
 * Copyright 2022 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
/**
 * @internal
 */
export declare const pierceQuerySelector: (root: Node, selector: string) => Element | null;
/**
 * @internal
 */
export declare const pierceQuerySelectorAll: (element: Node, selector: string) => Element[];
//# sourceMappingURL=PierceQuerySelector.d.ts.map