/**
 * @license
 * Copyright 2023 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
/**
 * @internal
 */
export declare const _isElementHandle: unique symbol;
//# sourceMappingURL=ElementHandleSymbol.d.ts.map