/**
 * @license
 * Copyright 2026 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
import type { Protocol } from 'devtools-protocol';
import type { Issue } from '../api/Issue.js';
/**
 * @internal
 */
export declare class CdpIssue implements Issue {
    #private;
    constructor(issue: Protocol.Audits.InspectorIssue);
    get code(): Protocol.Audits.InspectorIssueCode;
    get details(): Protocol.Audits.InspectorIssueDetails;
}
//# sourceMappingURL=CdpIssue.d.ts.map