/**
 * @license
 * Copyright 2020 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @internal
 */
export const PUPPETEER_REVISIONS = Object.freeze({
  chrome: '152.0.7977.54',
  'chrome-headless-shell': '152.0.7977.54',
  firefox: 'stable_154.0',
});
