/**
 * @license
 * Copyright 2023 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */

import type {CDPSession} from './CDPSession.js';
import type {Realm} from './Realm.js';

/**
 * @internal
 */
export interface Environment {
  get client(): CDPSession;
  mainRealm(): Realm;
}
