/**
 * @license
 * Copyright 2020 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
/**
 * @internal
 */
export declare class TaskQueue {
    #private;
    constructor();
    postTask<T>(task: () => Promise<T>): Promise<T>;
}
//# sourceMappingURL=TaskQueue.d.ts.map