/**
 * @license
 * Copyright 2026 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
/**
 * @internal
 */
export class CdpIssue {
    #code;
    #details;
    constructor(issue) {
        this.#code = issue.code;
        this.#details = issue.details;
    }
    get code() {
        return this.#code;
    }
    get details() {
        return this.#details;
    }
}
//# sourceMappingURL=CdpIssue.js.map