/**
 * @license
 * Copyright 2023 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
/**
 * @internal
 */
export declare function rm(path: string): Promise<void>;
/**
 * @internal
 */
export declare function rmSync(path: string): void;
//# sourceMappingURL=fs.d.ts.map