/**
 * @license
 * Copyright 2023 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
import { QueryHandler, type QuerySelector, type QuerySelectorAll } from './QueryHandler.js';
/**
 * @internal
 */
export declare class PQueryHandler extends QueryHandler {
    static querySelectorAll: QuerySelectorAll;
    static querySelector: QuerySelector;
}
//# sourceMappingURL=PQueryHandler.d.ts.map