/**
 * @license
 * Copyright 2026 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */

import type Protocol from 'devtools-protocol';

/**
 * The Issue interface represents a DevTools issue.
 *
 * @public
 */
export interface Issue {
  /**
   * The code of the issue.
   */
  code: string;

  /**
   * The details of the issue.
   */
  details: Protocol.Audits.InspectorIssueDetails;
}
