/**
 * @license
 * Copyright 2020 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
/**
 * @internal
 */
export declare const PUPPETEER_REVISIONS: Readonly<{
    chrome: "152.0.7977.54";
    'chrome-headless-shell': "152.0.7977.54";
    firefox: "stable_154.0";
}>;
//# sourceMappingURL=revisions.d.ts.map