/**
 * @license
 * Copyright 2023 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
import type * as Bidi from 'webdriver-bidi-protocol';
/**
 * @internal
 */
export declare class BidiSerializer {
    #private;
    static serialize(arg: unknown): Bidi.Script.LocalValue;
}
//# sourceMappingURL=Serializer.d.ts.map